(* $Id: $ *)

{
  open KeenParser
  exception Error
}

let digit = ['0'-'9']
let alphanum = ['a'-'z' 'A'-'Z' '0'-'9' '\'' '_']

rule token = parse
  (* Ignore spaces, tabs and carriage returns. *)
  | [' ' '\t' '\r']     { token lexbuf }
  (* Ignore comments. *)
  | '#' ([^'\n']*)      { token lexbuf }
  (* Update line count on new lines. *)
  | '\n'                { Lexing.new_line lexbuf ; TK_EOL }

  (* Keywords. *)
  | '+'                 { TK_ADD }
  | '-'                 { TK_SUB }
  | '*'                 { TK_MUL }
  | '/'                 { TK_DIV }
  | '='                 { TK_EQ }

  (* Natural numbers. *)
  | (digit)+ as s       { TK_INT(int_of_string s) }

  (* End of file. *)
  | eof                 { TK_EOF }

  (* Strings. *)
  | (alphanum)+ as s    { TK_STR(s) }

  (* Fail if nothing else matched. *)
  | _                   { raise Error }
