(* $Id: $ *)

%{
  module String = struct type t = string let compare = compare end
  module M = Map.Make (String)

  type op = Add | Sub | Mul | Div

  let add_cell map name cell =
    let cells = try M.find name map with Not_found -> []
    in
    M.add name (cell :: cells) map

  let add_row map i names =
    let (map', _) =
      List.fold_left
        (fun (map, j) name -> (add_cell map name (i, j), j+1))
        (map, 1)
        names
    in
    map'

  let keen_clue map (op, name, result) =
    let cells = try List.rev (M.find name map) with Not_found -> failwith "unknown name"
    in
    let kop =
      match op with
      | Add -> KeenSolver.Add cells
      | Sub ->
         begin
           match cells with
           | [c ; c'] -> KeenSolver.Sub (c, c')
           | _ -> failwith "operation `-' expects two arguments"
         end
      | Mul -> KeenSolver.Mul cells
      | Div ->
         begin
           match cells with
           | [c ; c'] -> KeenSolver.Div (c, c')
           | _ -> failwith "operation `/' expects two arguments"
         end
    in
    (kop, result)

  let output (n, m, map) clues =
    if n = m then
      (n,
       List.rev_map (keen_clue map) clues)
    else
      failwith "non-square grid"
%}

%token TK_ADD
%token TK_SUB
%token TK_MUL
%token TK_DIV
%token TK_EQ

%token <int> TK_INT
%token <string> TK_STR
%token TK_EOL
%token TK_EOF

(*
 * Returns a pair (size, clues) that is of type KeenSolver.game.
 *)
%start <KeenSolver.game> main

%%

main:
| list(TK_EOL) grid list(TK_EOL) clues list(TK_EOL) TK_EOF
  { output $2 $4 }

grid:
| grid TK_EOL nonempty_list(name)
  { let (n, m, map) = $1
    in
    if m = List.length $3 then
      let n' = n+1
      in
      let map' = add_row map n' $3
      in
      (n', m, map')
    else
      failwith "non-square grid" }
| nonempty_list(name)
  { (1, List.length $1, add_row M.empty 1 $1) }

name:
| TK_STR                { $1 }

clues:
| clues TK_EOL op name TK_EQ TK_INT
  { ($3, $4, $6) :: $1 }
| op name TK_EQ TK_INT
  { [($1, $2, $4)] }

%inline op:
| TK_ADD                { Add }
| TK_SUB                { Sub }
| TK_MUL                { Mul }
| TK_DIV                { Div }
