(* $Id $ *)

(* 
 * This is an implementation of a solver using Z3 for the game Keen (https://www.chiark.greenend.org.uk/~sgtatham/puzzles/js/keen.html) by Simon Tatham
 * 
 * Author: Vincent Penelle
 *)

open Z3

type cell = int * int
type op = Add of cell list | Sub of cell * cell | Mul of cell list | Div of cell * cell
type clue = op * int
type game = int * clue list

(* creates a Z3 context*)
let ctx = mk_context []

let game_formula game =
	failwith "Au boulot !"
	
	(* Takes a formula and returns the corresponding model, if it exists *)
let call_solver formula = 
	let solver = (Solver.mk_solver ctx (None)) in
	begin
	Solver.add solver [formula];
    (* Check satisfiability of the solver's assertions. *)
    let status = Solver.check solver [] in
    Format.printf "@[Solver says about the game: %s@]@." (Solver.string_of_status status);
		match status with
	| Z3.Solver.SATISFIABLE -> Solver.get_model solver 
	| Z3.Solver.UNKNOWN -> None
	| Z3.Solver.UNSATISFIABLE -> None 
	end
	
(* Function to print a row, given a model *)
let print_row row max_col model =
	let rec aux row col model =
			let x = Arithmetic.Integer.mk_const_s ctx ((string_of_int row)^","^(string_of_int col)) in
			let value = Model.get_const_interp_e model x in
			let s = 
				match value with
			| None -> "_"
			| Some y ->	string_of_int (Arithmetic.Integer.get_int y) in
			if (col = 1) then s else (aux row (col-1) model)^" "^s
	in
	
	Format.printf "@[%s\n@]@." (aux row max_col model)

let print_game game model =
	let rec aux row col model =
		if (row = 0) then Format.printf"@[Solution of the game:@]@."
								else begin 
											aux (row-1) col model;
										 print_row row col model
								end
	in
	
	match game with
	| (size,_) -> aux size size model