(* $Id $ *)

(* 
 * This is an implementation of a solver using Z3 for the game Unruly by Simon Tatham
 * 
 * Author: Vincent Penelle
 *)

open Z3

type clue = int * int
type game = int * int * clue list * clue list

(* creates a Z3 context*)
let ctx = mk_context []

(* Creates a formula representing a game *)
let game_formula game =
	failwith "Au Boulot!"
	
	
(* Takes a formula and returns the corresponding model, if it exists *)
let call_solver formula = 
	let solver = (Solver.mk_simple_solver ctx) in
	begin
	Solver.add solver [formula];
    (* Check satisfiability of the solver's assertions. *)
    let status = Solver.check solver [] in
    Format.printf "@[Solver says about the game: %s@]@." (Solver.string_of_status status);
	match status with
	| Solver.SATISFIABLE -> Solver.get_model solver 
	| Solver.UNKNOWN -> None
	| Solver.UNSATISFIABLE -> None
	end
	
			
(* Function to print a row, given a model *)
let print_row row max_col model =
	let rec aux row col model =
			let x = Boolean.mk_const_s ctx ((string_of_int row)^","^(string_of_int col)) in
			let value = Model.get_const_interp_e model x in
			let s = 
				match value with
			| None -> "_"
			| Some y ->	if Boolean.is_true y then "T" else "F" in
			if (col = 1) then s else (aux row (col-1) model)^" "^s
	in
	
	
	Format.printf "@[%s\n@]@." (aux row max_col model)

let print_game game model =
	let rec aux row col model =
		if (row = 0) then Format.printf"@[Solved Game:@]@."
								else begin 
											aux (row-1) col model;
										 print_row row col model
								end
	in
	
	match game with 
	| (row,col,_,_)  -> aux row col model
	
	