(* Helper function to print cells, i.e., pairs of integers. *)
let print_cell fmt (i, j) =
  Format.fprintf fmt "(%d,@ %d)" i j

(* Helper functions to print clues. *)
let print_op fmt =
  function
  | KeenSolver.Add cells ->
     Format.fprintf
       fmt "%a"
       (Print.list_printer_from_printer "@ +@ " print_cell)
       cells
  | KeenSolver.Sub (c, c') ->
     Format.fprintf
       fmt "%a@ -@ %a"
       print_cell c
       print_cell c'
  | KeenSolver.Mul cells ->
     Format.fprintf
       fmt "%a"
       (Print.list_printer_from_printer "@ *@ " print_cell)
       cells
  | KeenSolver.Div (c, c') ->
     Format.fprintf
       fmt "%a@ /@ %a"
       print_cell c
       print_cell c'
let print_clue fmt (op, res) =
  Format.fprintf fmt "@[%a@]@ =@ %d" print_op op res

(* Helper function to print input games. *)
let print_game fmt (size, clues) =
  Format.fprintf
    fmt
    "@[%d@ rows@ and@ %d@ columns@]@ @[<v 3>clues:@ %a@]"
    size size
    (Print.list_printer_from_printer "@ " print_clue)
    clues

(* Main. *)
let () =
  for i = 1 to (Array.length Sys.argv) - 1 do
    let file = Sys.argv.(i)
    in
    Format.printf "@[Input file:@ %s@]@." file ;
    let inchan = open_in file
    in
    let lexbuf = Lexing.from_channel inchan
    in
    let game =
      try
        KeenParser.main KeenLexer.token lexbuf
      with
      | KeenLexer.Error ->
         Format.eprintf
           "@[%s:@ line@ %d:@ lexical error@]@."
           file
           (Lexing.lexeme_start_p lexbuf).Lexing.pos_lnum ;
         exit 1
      | KeenParser.Error ->
         Format.eprintf
           "@[%s:@ line@ %d:@ syntax error@]@."
           file
           (Lexing.lexeme_start_p lexbuf).Lexing.pos_lnum ;
         exit 1
    in
    close_in inchan ;
    Format.printf "@[<v 3>Parsed game:@ %a@]@." print_game game ;
    let formula = KeenSolver.game_formula game
    in
    Format.printf "@[Formula of the game:@ %s@]@." (Z3.Expr.to_string formula) ;
    match KeenSolver.call_solver formula with
    | None -> Format.printf "@[This game has no solution@]@.";
    | Some x -> KeenSolver.print_game game x
  done
