(* Helper function to print cells, i.e., pairs of integers. *)
let print_cell fmt (i, j) =
  Format.fprintf fmt "(%d,@ %d)" i j

(* Helper function to print input games. *)
let print_game fmt (n, m, positive_clues, negative_clues) =
  let print_clue_list = Print.list_printer_from_printer ",@ " print_cell
  in
  Format.fprintf
    fmt
    "@[%d@ rows@]@ @[%d@ columns@]@ @[<hov 3>positive_clues:@ %a@]@ @[<hov 3>negative_clues:@ %a@]"
    n m
    print_clue_list positive_clues
    print_clue_list negative_clues

(* Main. *)
let () =
  for i = 1 to (Array.length Sys.argv) - 1 do
    let file = Sys.argv.(i)
    in
    Format.printf "@[Input file:@ %s@]@." file ;
    let inchan = open_in file
    in
    let lexbuf = Lexing.from_channel inchan
    in
    let game =
      try
        UnrulyParser.main UnrulyLexer.token lexbuf
      with
      | UnrulyLexer.Error ->
         Format.eprintf
           "@[%s:@ line@ %d:@ lexical error@]@."
           file
           (Lexing.lexeme_start_p lexbuf).Lexing.pos_lnum ;
         exit 1
      | UnrulyParser.Error ->
         Format.eprintf
           "@[%s:@ line@ %d:@ syntax error@]@."
           file
           (Lexing.lexeme_start_p lexbuf).Lexing.pos_lnum ;
         exit 1
    in
    close_in inchan ;
    Format.printf "@[<v 3>Parsed game:@ %a@]@." print_game game ;
    let formula = UnrulySolver.game_formula game
    in
    Format.printf "@[Formula of the game:@ %s@]@." (Z3.Expr.to_string formula) ;
    match UnrulySolver.call_solver formula with
    | None -> Format.printf "@[This game has no solution@]@.";
    | Some x -> UnrulySolver.print_game game x
  done
